/*
 
 Copyright (C) 2010,2011 Timo Lassmann <timolassmann@gmail.com>
 
 This file is part of delve.
 
 Delve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Delve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Delve.  If not, see <http://www.gnu.org/licenses/>.
 
 */

#define NCODE  12
#define IGAP 13
#define EGAP 14
#define BORDER_REGION 2
#define PSEUDO_CONSTANT 1000
#define MAX_LOCAL_POSTERIORS 100000

struct hmm2{ 
	
	
	float** fM;
	float** fX;
	float** fY;
	
	float*** tfM;
	float*** tfX;
	float*** tfY;
	
	float** bM;
	float** bX;
	float** bY;
	
	unsigned short int** traceback;
	
	float* prob;
	float* tprob;
	
	float genome_score;
	float random_score;
	float score;
	float* random_genome_model; 
	float* random_read_model;
	float* random_genome_model_t; 
	float* random_read_model_t;
	int* p_value_ranks;
	int train_count;
	int x;
	int y;
};

struct thread_data2{
	struct read_info** ri;
	struct hmm2* hmm;
	struct db* db;
	struct rescue_probabilities* rp;
	struct parameters* param;
	struct genome_probabilities* gp;
	
	int* p_value_ranks;
	
	int start;
	int end;
};


struct posteriors_collector {
	unsigned int strand;
	unsigned int pos;
	unsigned int len;
	float* local_posteriors;
};


struct hmm2* get_fdr_cutoff(struct hmm2* hmm,struct read_info** ri,struct db* db,struct parameters* param,int size);
void* do_get_fdr_thread_hmm2(void *threadarg);

void *functionC();
int phmm2_posterior_controller(struct db* db,struct parameters* param);
struct hmm2* run_posterior2(struct hmm2* localhmm,struct read_info** ri,struct db* db,struct genome_probabilities* gp,struct parameters* param,int size);
struct hmm2* random_model_calc(struct hmm2* hmm, unsigned char* rseq, unsigned char* gseq,int rlen, int glen,float frac,int mode);
void* do_get_posteriors_thread2(void *threadarg);


struct hmm2* score_genome(struct hmm2* hmm, unsigned char* seq,int len);
struct hmm2* get_genome_prob(struct hmm2* hmm, unsigned char* seq,int len, float frac);

int phmm2_controller(struct db* db,struct parameters* param);
struct hmm2* run_pHMM2(struct hmm2* localhmm,struct read_info** ri,struct db* db,struct parameters* param,int size);
struct hmm2* run_pHMM2_random(struct hmm2* localhmm,struct read_info** ri,struct db* db,struct parameters* param,int size);
void* do_baum_welch_thread2(void *threadarg);
void* do_baum_welch_thread2_random_model(void *threadarg);

struct read_info** run_score_alignments_hmm2(struct hmm2* hmm,struct read_info** ri,struct db* db,struct parameters* param,int size);
void* do_score_alignments_thread_hmm2(void *threadarg);

float* read_genome_posteriors(struct hmm2* hmm,unsigned char* a, unsigned char* b, int n,int m, float frac,int count_mismatches_posterior);
unsigned char* glocal_viterbi_2ndorder_log_Y(struct hmm2* hmm,unsigned char* a, unsigned char* b, int n,int m);
struct hmm2* get_prob_2ndorder_log_Y(struct hmm2* hmm,unsigned char* a, unsigned char* b, int n,int m,float frac, int frame);
struct hmm2* glocal_forward_2ndorder_log_Y(struct hmm2* hmm,unsigned char* a, unsigned char* b, int n,int m,int frame);
struct hmm2* glocal_backward_2ndorder_log_Y(struct hmm2* hmm,unsigned char* a, unsigned char* b, int n,int m);
struct hmm2* re_estimate_2ndorder(struct hmm2* hmm);
struct hmm2* re_estimate_2ndorder_random(struct hmm2* hmm);



struct hmm2* init_2nd_hmm(int x, int y);
void free_2nd_hmm(struct hmm2* hmm);


void write_hmm2_to_file(struct hmm2* hmm,struct parameters * param);
void read_hmm2_from_file(struct hmm2* hmm,struct parameters * param);

void init_thread_hmm2s(struct thread_data2* thread_data_array,struct hmm2* localhmm,int numthreads);
void entangle_hmm2s(struct thread_data2* thread_data_array,struct hmm2* localhmm,int numthreads);

void simple_print_path(unsigned char* alignment);
unsigned char* reverse_path(unsigned char* org );
unsigned char* convert_16_to_5_nuc(unsigned char*  seq, int len);
unsigned char* reverse_complement_16(unsigned char* p,int len);
unsigned char* transform_genome_sequence(unsigned char* seq,unsigned int len);
float* reverse_genome_posteriors(float* a, int len);


int get_alignment_offset(unsigned char* org);
void write_rescue_probabilities(struct rescue_probabilities* rp, unsigned int len,unsigned int block_len,struct parameters * param);
void read_rescue_probabilities(struct rescue_probabilities* rp, unsigned int len,unsigned int block_len,struct parameters * param);




